/*
 *  arch/ppc/mpc5xxx_io/sdma.c
 *
 *  Driver for MPC5xxx processor SmartComm peripheral controller
 *
 *  Author: Dale Farnsworth <dfarnsworth@mvista.com>
 *
 * 2003 (c) MontaVista, Software, Inc.  This file is licensed under the terms
 * of the GNU General Public License version 2.  This program is licensed
 * "as is" without any warranty of any kind, whether express or implied.
 */

#include <asm/io.h>
#include <asm/mpc5xxx.h>
#include <asm/delay.h>
#include <bestcomm_api.h>
#include <mgt5200/sdma.h>

#define RFIFO_DATA	0xf0003184
#define TFIFO_DATA	0xf00031a4

/*
 * returns 1 on success
 * returns 0 if already loaded
 * returns -1 on failure
 */
int
mpc5xxx_sdma_load_tasks_image(void)
{
	struct mpc5xxx_sdma *sdma = (struct mpc5xxx_sdma *)MPC5xxx_SDMA;
	int task;
	static int done = 0;

	if (done)
		return 0;

	/* zero some registers before we start */
	for (task=0; task<MPC5xxx_SDMA_MAX_TASKS; task++) {
		out_be16(&sdma->tcr_0 + task, 0);
		out_8(&sdma->IPR0 + task, 0);
	}

	out_8(&sdma->IPR0, 7);	/* always */
	out_8(&sdma->IPR3, 6);	/* eth rx */
	out_8(&sdma->IPR4, 5);	/* eth tx */

	TasksInitAPI((uint8 *) MPC5xxx_MBAR);
	TasksLoadImage((sdma_regs *)MPC5xxx_SDMA);
	done = 1;
	return 1;
}

/*
 * Initialize FEC receive task.
 * Returns task number of FEC receive task.
 * Returns -1 on failure
 */
int
mpc5xxx_sdma_fec_rx_task_setup(int num_bufs, int maxbufsize)
{
	struct mpc5xxx_sdma *sdma = (struct mpc5xxx_sdma *)MPC5xxx_SDMA;
	static TaskSetupParamSet_t params;
	int tasknum;

	if (mpc5xxx_sdma_load_tasks_image() < 0)
		return -1;

	params.NumBD = num_bufs;
	params.Size.MaxBuf = maxbufsize;
	params.StartAddrSrc = RFIFO_DATA;
	params.IncrSrc = 0;
	params.SzSrc = 4;
	params.IncrDst = 4;
	params.SzDst = 4;

	tasknum = TaskSetup(TASK_FEC_RX, &params);

	/* clear pending interrupt bits */
	out_be32(&sdma->IntPend, 1<<tasknum);

	return tasknum;
}

/*
 * Initialize FEC transmit task.
 * Returns task number of FEC transmit task.
 * Returns -1 on failure
 */
int
mpc5xxx_sdma_fec_tx_task_setup(int num_bufs)
{
	struct mpc5xxx_sdma *sdma = (struct mpc5xxx_sdma *)MPC5xxx_SDMA;
	static TaskSetupParamSet_t params;
	int tasknum;

	if (mpc5xxx_sdma_load_tasks_image() < 0)
		return -1;

	params.NumBD = num_bufs;
	params.IncrSrc = 4;
	params.SzSrc = 4;
	params.StartAddrDst = TFIFO_DATA;
	params.IncrDst = 0;
	params.SzDst = 4;

	tasknum = TaskSetup(TASK_FEC_TX, &params);

	/* clear pending interrupt bits */
	out_be32(&sdma->IntPend, 1<<tasknum);

	return tasknum;
}

void
mpc5xxx_sdma_enable_task(int task)
{
	struct mpc5xxx_sdma *sdma = (struct mpc5xxx_sdma *)MPC5xxx_SDMA;
	u16 val16;

	val16 = in_be16(&sdma->tcr_0 + task);
	val16 |= MPC5xxx_SDMA_TASK_ENABLE;
	out_be16(&sdma->tcr_0 + task, val16);
}

void
mpc5xxx_sdma_disable_task(int task)
{
	struct mpc5xxx_sdma *sdma = (struct mpc5xxx_sdma *)MPC5xxx_SDMA;
	u16 val16;

	val16 = in_be16(&sdma->tcr_0 + task);
	val16 &= ~MPC5xxx_SDMA_TASK_ENABLE;
	out_be16(&sdma->tcr_0 + task, val16);
}
