/*
 *
 * Author: Dale Farnsworth <dale.farnsworth@mvista.com>
 *
 * 2003 (c) MontaVista Software Inc.
 *
 * 2003 (C) Wolfgang Denk, DENX Software Engineering, wd@denx.de.
 *
 * This program is free software; you can redistribute  it and/or modify it
 * under  the terms of  the GNU General Public License as published by the
 * Free Software Foundation;  either version 2 of the  License, or (at your
 * option) any later version.
 */

#include <linux/config.h>
#include <asm/mpc5xxx.h>
#include <asm/time.h>

static int cpu_5xxx[] = {
	0,  0,  0,  10, 20, 20, 25, 45,
	30, 55, 40, 50, 0,  60, 35, 0,
	30, 25, 65, 10, 70, 20, 75, 45,
	0,  55, 40, 50, 80, 60, 35, 0
};

static int cpufreq;
static int busfreq;
static int ipbfreq;
static int pcifreq;

/* useful macros for manipulating CSx_START/STOP */
#if defined(CONFIG_MPC5100)
#define START_REG(start)	((start) >> 15)
#define STOP_REG(start, size)	(((start) + (size) - 1) >> 15)
#elif defined(CONFIG_MPC5200)
#define START_REG(start)	((start) >> 16)
#define STOP_REG(start, size)	(((start) + (size) - 1) >> 16)
#endif

void
mpc5xxx_cpu_init(void)
{
    	u32 addecr = 0;
	struct mpc5xxx_xlb *xlb = (struct mpc5xxx_xlb *) MPC5xxx_XLB;

	*(u32 *) MPC5xxx_BOOTCS_START = START_REG(CONFIG_PPC_5xxx_FLASH_ADDR);
	*(u32 *) MPC5xxx_BOOTCS_STOP = STOP_REG(CONFIG_PPC_5xxx_FLASH_ADDR,
			CONFIG_PPC_5xxx_FLASH_SIZE);
	addecr |= (1 << 25); /* Boot_CS */

	/* enable writes to flash so MTD probing will work */
	*(u32 *) MPC5xxx_BOOTCS_CFG &= ~1;

#if defined(CONFIG_MPC5200)
	/* Enable IPB wait state */
	addecr |= (1 << 0);

	*(u32 *) MPC5xxx_CS_BURST = 0x00000000;
	*(u32 *) MPC5xxx_CS_DEADCYCLE = 0x33333333;
#endif
	/* Enable chip selects */
	*(u32 *) MPC5xxx_ADDECR  = addecr;
	*(u32 *) MPC5xxx_CS_CTRL = (1 << 24);

	/* Setup pin multiplexing */
	*(u32 *) MPC5xxx_GPS_PORT_CONFIG  = MPC5xxx_GPIO_PORT_CONFIG;

#if defined(CONFIG_MPC5200)
	/* Enable timebase */
	xlb->config |= (1 << 13);

	/* Configure the XLB Arbiter */
	xlb->master_pri_enable = 0xff;
	xlb->master_priority = 0x11111111;
#endif
}

void
mpc5xxx_restart(char *cmd)
{
	__cli();

	/* Turn on watchdog to reset the machine */
	*(u32 *) MPC5xxx_GPT0_COUNTER = 0xf;
	*(u32 *) MPC5xxx_GPT0_ENABLE = 0x9004;

	while (1);
}

void
mpc5xxx_halt(void)
{
	__cli();
	while (1);
}

void
mpc5xxx_power_off(void)
{ 
	mpc5xxx_halt();
}

/*
 * Set BAT 3 to map 0xf0000000 to end of physical memory space.
 */
void __init
mpc5xxx_set_bat(void)
{
	mb();
	mtspr(DBAT2U, 0xf0001ffe);
	mtspr(DBAT2L, 0xf000002a);
	mb();
}

int
mpc5xxx_cpufreq(void)
{
	return cpufreq;
}

int
mpc5xxx_busfreq(void)
{
	return busfreq;
}

int
mpc5xxx_ipbfreq(void)
{
	return ipbfreq;
}

int
mpc5xxx_pcifreq(void)
{
	return pcifreq;
}

/* The decrementer counts at the system (internal) clock frequency
 * divided by four.  The most accurate time base is connected to the
 * rtc.  We read the decrementer change during one rtc tick (one second)
 * and multiply by 4 to get the system clock frequency.
 */
void __init
mpc5xxx_calibrate_decr(void)
{
	int freq, divisor;
	struct mpc5xxx_rtc *rtc = (struct mpc5xxx_rtc*)MPC5xxx_RTC;
	struct mpc5xxx_cdm *cdm = (struct mpc5xxx_cdm*)MPC5xxx_CDM;
	int current_time, previous_time;
	int tbl_start, tbl_end;

	previous_time = in_be32(&rtc->time);
	while ((current_time = in_be32(&rtc->time)) == previous_time) ;
	tbl_start = get_tbl();
	previous_time = current_time;
	while ((current_time = in_be32(&rtc->time)) == previous_time) ;
	tbl_end = get_tbl();

	freq = (tbl_end - tbl_start) * 4;

	divisor = 4;
	tb_ticks_per_jiffy = freq / HZ / divisor;
	tb_to_us = mulhwu_scale_factor(freq / divisor, 1000000);

	busfreq = freq;
	cpufreq = busfreq / 10 * cpu_5xxx[in_be32(&cdm->rstcfg) & 0x1f];
	ipbfreq = (in_8(&cdm->ipb_clk_sel) & 1) ? busfreq / 2 : busfreq;
	switch (in_8(&cdm->pci_clk_sel) & 3) {
	case 0:
	    	pcifreq = ipbfreq;
		break;
	case 1:
		pcifreq = ipbfreq / 2;
		break;
	default:
		pcifreq = ipbfreq / 4;
		break;
	}
	printk("frequencies: cpu=%d, bus=%d, ipb=%d, pci=%d\n",
		cpufreq, busfreq, ipbfreq, pcifreq);
}

#ifdef  CONFIG_SERIAL_TEXT_DEBUG
#if   CONFIG_PPC_5xxx_PSC_CONSOLE_PORT == 0
#define MPC5xxx_CONSOLE	MPC5xxx_PSC1
#elif CONFIG_PPC_5xxx_PSC_CONSOLE_PORT == 1
#define MPC5xxx_CONSOLE	MPC5xxx_PSC2
#elif CONFIG_PPC_5xxx_PSC_CONSOLE_PORT == 2
#define MPC5xxx_CONSOLE	MPC5xxx_PSC3
#else
#error "mpc5xxx PSC for console not selected"
#endif
void
mpc5xxx_progress(char *s, unsigned short hex)
{
    	struct mpc5xxx_psc *psc = (struct mpc5xxx_psc *)MPC5xxx_CONSOLE;
	char c;

	while ((c = *s++) != 0) {
		if (c == '\n') {
			while (!(in_be16(&psc->mpc5xxx_psc_status) &
					MPC5xxx_PSC_SR_TXRDY)) ;
			out_8(&psc->mpc5xxx_psc_buffer_8, '\r');
		}
		while (!(in_be16(&psc->mpc5xxx_psc_status) &
			    	MPC5xxx_PSC_SR_TXRDY)) ;
		out_8(&psc->mpc5xxx_psc_buffer_8, c);
	}
}
#endif	/* CONFIG_SERIAL_TEXT_DEBUG */

unsigned long __init
mpc5xxx_find_end_of_memory(void)
{
#if defined(CONFIG_MPC5100)
    	unsigned long last_sdram_block = in_be32((u32 *)MPC5xxx_SDRAM_STOP);

	return MPC5xxx_SDRAM_UNIT * (last_sdram_block + 1);
#elif defined(CONFIG_MPC5200)
    	u32 sdram_config_0 = in_be32((u32 *)MPC5xxx_SDRAM_CONFIG_0);
    	u32 sdram_config_1 = in_be32((u32 *)MPC5xxx_SDRAM_CONFIG_1);
	u32 ramsize = 0;

	if ((sdram_config_0 & 0x1f) >= 0x13)
		ramsize = 1 << ((sdram_config_0 & 0xf) + 17);

	if (((sdram_config_1 & 0x1f) >= 0x13) &&
			((sdram_config_1 & 0xfff00000) == ramsize))
		ramsize += 1 << ((sdram_config_1 & 0xf) + 17);

	return ramsize;
#endif
}

void __init
mpc5xxx_map_io(void)
{
	io_block_mapping(0x50000000, 0x50000000, 0x01000000, _PAGE_IO);
	io_block_mapping(0xf0000000, 0xf0000000, 0x10000000, _PAGE_IO);
}

#ifdef CONFIG_VT

#define SC_LIM 89

/* Simple translation table for the SysRq keys */

#ifdef CONFIG_MAGIC_SYSRQ

unsigned char mpc5xxx_sysrq_xlate[128] =
"\000\0331234567890-=\177\t"			/* 0x00 - 0x0f */
"qwertyuiop[]\r\000as"				/* 0x10 - 0x1f */
"dfghjkl;'`\000\\zxcv"				/* 0x20 - 0x2f */
"bnm,./\000*\000 \000\201\202\203\204\205"	/* 0x30 - 0x3f */
"\206\207\210\211\212\000\000789-456+1"		/* 0x40 - 0x4f */
"230\177\000\000\213\214\000\000\000\000\000\000\000\000\000\000" /* 0x50 - 0x5f */
"\r\000/";					/* 0x60 - 0x6f */

#endif


int mpc5xxx_setkeycode(unsigned int scancode, unsigned int keycode)
{
  return -EINVAL;
}

int mpc5xxx_getkeycode(unsigned int scancode)
{
  return -EINVAL;
}

int mpc5xxx_translate(unsigned char scancode, unsigned char *keycode,
			   char raw_mode)
{
  //0xFF is sent by a few keyboards, ignore it. 0x00 is error 
  if (scancode == 0x00 || scancode == 0xff) {
    return 0;
  }
  *keycode = scancode;
  return 1;
}

char mpc5xxx_unexpected_up(unsigned char keycode)
{
  /* unexpected, but this can happen: maybe this was a key release for a
     FOCUS 9000 PF key; if we want to see it, we have to clear up_flag */
  if (keycode >= SC_LIM || keycode == 85)
    return 0;
  else
    return 0200;
}

void mpc5xxx_kbd_init_hw(void) 
{
  return ;
}

#endif
