/*
 * $Id: wios.c,v 1.1 2005/07/07 21:00:45 andyg Exp $
 *
 * Normal mappings of chips in physical memory
 *
 * Modified for Netgear GSM73xx partitions... -- JWL
 */

/* partition support */
#define HAVE_PARTITIONS

#include <linux/module.h>
#include <linux/types.h>
#include <linux/kernel.h>
#include <asm/io.h>
#include <linux/mtd/mtd.h>
#include <linux/mtd/concat.h>
#include <linux/mtd/map.h>
#include <linux/config.h>
#include <linux/init.h>
#ifdef HAVE_PARTITIONS
#include <linux/mtd/partitions.h>
#endif

/* bank0-bank1 have 8-bit data bus */
#define BANK0_WINDOW_ADDR      0xff800000
#define BANK0_WINDOW_SIZE      0x00800000

#define BANK1_WINDOW_ADDR      0xff000000
#define BANK1_WINDOW_SIZE      0x00800000

/* bank2-bank5 have 16-bit data bus */
#define BANK2_WINDOW_ADDR      0x74000000
#define BANK2_WINDOW_SIZE      0x01000000

#define BANK3_WINDOW_ADDR      0x75000000
#define BANK3_WINDOW_SIZE      0x01000000

#define BANK4_WINDOW_ADDR      0x76000000
#define BANK4_WINDOW_SIZE      0x01000000

#define BANK5_WINDOW_ADDR      0x77000000
#define BANK5_WINDOW_SIZE      0x01000000

#define WINDOW_ADDR	0xff000000
#define WINDOW_SIZE8	0x00800000
#define WINDOW_SIZE16   0x01000000
#define BUSWIDTH8	1
#define BUSWIDTH16	2

#define APP_START	0x00000000
#define APP_LEN		0x005a0000

#define SYSTEM_START	0x005a0000
#define SYSTEM_LEN	0x00160000

#define U_BOOT_START	0x00700000
#define U_BOOT_LEN	0x00020000

#define ENV_START	0x00720000
#define ENV_LEN		0x00020000

#define VPD_START	0x00740000
#define VPD_LEN		0x00020000

#define SYSTEM_2_START	0x00760000
#define SYSTEM_2_LEN	0x000a0000

#define MAX_NUM_FLASH_BANKS     6
static struct mtd_info *mymtd[MAX_NUM_FLASH_BANKS] = { NULL, };
static struct mtd_info *concat = NULL;

#ifdef HAVE_PARTITIONS
static struct mtd_partition gsm73xx_partitions[] =
{
	/* app filesystem */
	{
		      name: "Application Filesystem",
		    offset: APP_START,
		      size: APP_LEN,
		mask_flags: 0
	},
	/* system (kernel + ramdisk) */
	{
		      name: "System (Kernel + Ramdisk)",
		    offset: SYSTEM_START,
		      size: SYSTEM_LEN,
		mask_flags: 0
	},
	/* u-boot */
	{
		      name: "U-Boot",
		    offset: U_BOOT_START,
		      size: U_BOOT_LEN,
		mask_flags: 0
	},
	/* u-boot environment */
	{
		      name: "U-Boot Environment",
		    offset: ENV_START,
		      size: ENV_LEN,
		mask_flags: 0
	},
	/* VPD */
	{
		      name: "FASTPATH Vital Product Data",
		    offset: VPD_START,
		      size: VPD_LEN,
		mask_flags: 0
	},
	/* system (part 2) */
	{
		      name: "System (part 2)",
		    offset: SYSTEM_2_START,
		      size: SYSTEM_2_LEN,
		mask_flags: 0
	}
};

#define NUM_PARTITIONS	(sizeof(gsm73xx_partitions) \
                       / sizeof(struct mtd_partition))
#endif

/* 8M, 8-bit access */
struct map_info gsm73xx_map_bank0 = {
	name: "Bank 0 - 8-bit bus",
	size: BANK0_WINDOW_SIZE,
	buswidth: BUSWIDTH8,
        virt: BANK0_WINDOW_ADDR
                           
};

struct map_info gsm73xx_map_bank1 = {
	name: "Bank 1 - 8-bit bus",
	size: BANK1_WINDOW_SIZE,
	buswidth: BUSWIDTH8,
        virt: BANK1_WINDOW_ADDR
                           
};
/* 8M, 16-bit access */
struct map_info gsm73xx_map_bank2 = {
	name: "Bank 2 - 16-bit bus",
	size: BANK2_WINDOW_SIZE,
	buswidth: BUSWIDTH16,
        virt: BANK2_WINDOW_ADDR
};

/* 16M, 16-bit access */
struct map_info gsm73xx_map_bank3 = {
	name: "Bank 3 - 16-bit bus",
	size: BANK3_WINDOW_SIZE,
	buswidth: BUSWIDTH16,
        virt: BANK3_WINDOW_ADDR
};

/* 16M, 16-bit access */
struct map_info gsm73xx_map_bank4 = {
	name: "Bank 4 - 16-bit bus",
	size: BANK4_WINDOW_SIZE,
	buswidth: BUSWIDTH16,
        virt: BANK4_WINDOW_ADDR
};
/* 16M, 16-bit access */
struct map_info gsm73xx_map_bank5 = {
	name: "Bank 5 - 16-bit bus",
	size: BANK5_WINDOW_SIZE,
	buswidth: BUSWIDTH16,
        virt: BANK5_WINDOW_ADDR
};

int __init init_gsm73xx(void)
{
#if 1  
       int num_devices = 0;
       ulong flash_storage = 0;

       /* map the devices */

       gsm73xx_map_bank2.virt = (unsigned long)ioremap(BANK2_WINDOW_ADDR, BANK2_WINDOW_SIZE);
       if (!gsm73xx_map_bank2.virt) {
		printk("Failed to ioremap\n");
		return -EIO;
       }

       gsm73xx_map_bank3.virt = (unsigned long)ioremap(BANK3_WINDOW_ADDR, BANK3_WINDOW_SIZE);
       if (!gsm73xx_map_bank3.virt) {
		printk("Failed to ioremap\n");
		return -EIO;
       }

       gsm73xx_map_bank4.virt = (unsigned long)ioremap(BANK4_WINDOW_ADDR, BANK4_WINDOW_SIZE);
       if (!gsm73xx_map_bank4.virt) {
		printk("Failed to ioremap\n");
		return -EIO;
       }

       gsm73xx_map_bank5.virt = (unsigned long)ioremap(BANK5_WINDOW_ADDR, BANK5_WINDOW_SIZE);
       if (!gsm73xx_map_bank5.virt) {
		printk("Failed to ioremap\n");
		return -EIO;
       }
       /* here we will need to detect how many devices are present and then adjust the 
        * partition offsets accordingly */
       
       /* second won't be there most of the time */
       mymtd[num_devices] = do_map_probe("cfi_probe",&gsm73xx_map_bank1);
       if (mymtd[num_devices]) {
         mymtd[num_devices]->owner = THIS_MODULE;
         flash_storage += mymtd[num_devices]->size;
         num_devices++;
       }
       
       /* this one should be 8M */
       mymtd[num_devices] = do_map_probe("cfi_probe",&gsm73xx_map_bank2);
       if (mymtd[num_devices]) {
         mymtd[num_devices]->owner = THIS_MODULE;
         flash_storage += mymtd[num_devices]->size;
         num_devices++;
       }

       /* 16M for the rest */
       mymtd[num_devices] = do_map_probe("cfi_probe",&gsm73xx_map_bank3);
       if (mymtd[num_devices]) {
         mymtd[num_devices]->owner = THIS_MODULE;
         flash_storage += mymtd[num_devices]->size;
         num_devices++;
       }
       
       mymtd[num_devices] = do_map_probe("cfi_probe",&gsm73xx_map_bank4);
       if (mymtd[num_devices]) {
         mymtd[num_devices]->owner = THIS_MODULE;
         flash_storage += mymtd[num_devices]->size;
         num_devices++;
       }

       mymtd[num_devices] = do_map_probe("cfi_probe",&gsm73xx_map_bank5);
       if (mymtd[num_devices]) {
         mymtd[num_devices]->owner = THIS_MODULE;
         flash_storage += mymtd[num_devices]->size;
         num_devices++;
       }

       /* first bank should always be there, but needs to be last in the list */
       mymtd[num_devices] = do_map_probe("cfi_probe",&gsm73xx_map_bank0);
       if (mymtd[num_devices]) {
         mymtd[num_devices]->owner = THIS_MODULE;
         num_devices++;
       }

       concat = mtd_concat_create(mymtd, num_devices, "WIOS Flash");

       if (concat){

         /* as long as everything got created, we now need to adjust the offset 
          * depending on how many extra banks were found */

#if defined(HAVE_PARTITIONS)

         /* first partition is all for application */
         gsm73xx_partitions[0].size += flash_storage;
         /* second one is for system 1/2 */
         gsm73xx_partitions[1].offset += flash_storage;
         /* second one is for uboot */
         gsm73xx_partitions[2].offset += flash_storage;
         /* second one is for uboot env */
         gsm73xx_partitions[3].offset += flash_storage;
         /* second one is for vpd */
         gsm73xx_partitions[4].offset += flash_storage;
         /* second one is for system 2/2 */
         gsm73xx_partitions[5].offset += flash_storage;

         add_mtd_partitions(concat, gsm73xx_partitions, NUM_PARTITIONS);

#else
         add_mtd_device(concat);
#endif
         return 0;

       }

       return -ENXIO;

#else
       	printk(KERN_NOTICE "gsm73xx flash device: %x at %x\n", WINDOW_SIZE, WINDOW_ADDR);
	gsm73xx_map.virt = (unsigned long)ioremap(WINDOW_ADDR, WINDOW_SIZE);

	if (!gsm73xx_map.virt) {
		printk("Failed to ioremap\n");
		return -EIO;
	}
	mymtd = do_map_probe("cfi_probe", &gsm73xx_map);
	if (mymtd) {
		mymtd->owner = THIS_MODULE;

#if defined(HAVE_PARTITIONS)
		add_mtd_partitions(mymtd, gsm73xx_partitions,
		                   NUM_PARTITIONS);
#else
		add_mtd_device(mymtd);
#endif
		return 0;
	} else { /* Some boxes have 1/2 flash... */
		iounmap((void *)gsm73xx_map.virt);
		gsm73xx_map.virt
		   = (unsigned long)ioremap(WINDOW_ADDR + (WINDOW_SIZE / 2),
		                            (WINDOW_SIZE / 2));
		gsm73xx_map.size = (WINDOW_SIZE / 2);

		mymtd = do_map_probe("cfi_probe", &gsm73xx_map);
		if (mymtd) {
			mymtd->owner = THIS_MODULE;

#if defined(HAVE_PARTITIONS)
			add_mtd_partitions(mymtd, gsm73xx_partitions_2,
			                   NUM_PARTITIONS);
#else
			add_mtd_device(mymtd);
#endif
			return 0;
		}
	}

	iounmap((void *)gsm73xx_map.virt);
	return -ENXIO;
#endif 
}

static void __exit cleanup_gsm73xx(void)
{
	if (mymtd) {
#if defined(HAVE_PARTITIONS)
		del_mtd_device(concat);
#else
		del_mtd_partitions(mymtd);
#endif
		map_destroy(mymtd[0]);
		map_destroy(mymtd[1]);
		map_destroy(mymtd[2]);
		map_destroy(mymtd[3]);
		map_destroy(mymtd[4]);
		map_destroy(mymtd[5]);
	}
#if 0
	if (gsm73xx_map.virt) {
		iounmap((void *)gsm73xx_map.virt);
		gsm73xx_map.virt = 0;
	}
#endif 
}

module_init(init_gsm73xx);
module_exit(cleanup_gsm73xx);

MODULE_LICENSE("GPL");
MODULE_AUTHOR("Andy Gospodarek <andyg@lvl7.com>");
MODULE_DESCRIPTION("MTD map driver Alcatel WIOS");
