#ifdef CONFIG_BERT

#ifndef GLUEBERT_H
#define GLUEBERT_H

/***********************************************************
 * Library file for GlueBert Part.
 * These registers reside in the 256MB extended ROM Space.
 *
 * Jason Yee, 12/6/02
 ***********************************************************/


/***********************************************************************
 * Misc
 **********************************************************************/
#define MISC_REGS_BASE               0x78000000

#define MISC_REV_REG_ID              REG_ID_001
#define MISC_REV_REG_OFFSET          0x00  /* Read Only,  8 bits wide */
#define MISC_REV_MINOR_MASK          0x0F
#define MISC_REV_MAJOR_MASK          0xF0
#define MISC_REV_REG_MASK            (MISC_REV_MAJOR_MASK | MISC_REV_MINOR_MASK)
#define MISC_REV_REG_RST_VAL         0x01

#define MISC_TEST_REG_ID             REG_ID_002
#define MISC_TEST_REG_OFFSET         0x04  /* Read/Write, 8 bits wide */
#define MISC_TEST_REG_MASK           0xFF

#define MISC_POWER_LED_REG_ID        REG_ID_003
#define MISC_POWER_LED_OFFSET        0x08
#define POWER_LED_OFF                0x3
#define POWER_LED_GREEN              0x2
#define POWER_LED_AMBER              0x1
#define POWER_LED_NOTEST             0x0
#define MISC_POWER_LED_MASK          (POWER_LED_NOTEST | \
                                      POWER_LED_GREEN  | \
                                      POWER_LED_AMBER  | \
                                      POWER_LED_OFF)

#define MISC_FC_MEZ_ID_REG_ID        REG_ID_004
#define MISC_FC_MEZ_ID_OFFSET        0x0c
#define FC_MEZ_ID0                   0x1
#define FC_MEZ_ID1                   0x2
#define FC_MEZ_ID2                   0x4
#define MEZ_10G_PRESENT              0x8
#define MEZ_ERNIE_PRESENT            0x10
#define MISC_FC_MEZ_ID_MASK          (FC_MEZ_ID0 | \
                                      FC_MEZ_ID1 | \
                                      FC_MEZ_ID2 | \
                                      MEZ_10G_PRESENT | \
                                      MEZ_ERNIE_PRESENT)

#define MISC_FLASH_STATUS_REG_ID     REG_ID_005
#define MISC_FLASH_STATUS_OFFSET     0x10
#define FLASH_PROMICE_DISABLE        0x1
#define FLASH_FLSH2_BUSY             0x2
#define FLASH_FLSH1_BUSY             0x4
#define FLASH_REG_MASK               (FLASH_PROMICE_DISABLE | \
                                      FLASH_FLSH2_BUSY | \
                                      FLASH_FLSH1_BUSY)

#define MISC_NOT_USED_REG_ID         REG_ID_006
#define MISC_NOT_USED_OFFSET         0x14
#define MISC_NOT_USED_MASK           0xFF

#define MISC_FAULT_STATUS_REG_ID     REG_ID_007
#define MISC_FAULT_STATUS_OFFSET     0x18
#define MP_FAULT_OCCURED             0x1
#define MISC_FAULT_STATUS_MASK       (MP_FAULT_OCCURED)

#define MISC_COMMAND_REG_ID          REG_ID_008
#define MISC_COMMAND_REG_OFFSET      0x1c
#define MISC_COMMAND_REG_MASK        0xff

#define MISC_WATCHDOG_CNT_REG_ID     REG_ID_009
#define MISC_WATCHDOG_CNT_OFFSET     0x20
#define MISC_WATCHDOG_CNT_MASK       0xFF

#define MISC_WATCHDOG_DIS_REG_ID     REG_ID_010
#define MISC_WATCHDOG_DIS_OFFSET     0x24
#define MISC_WATCHDOG_DISD           0x59
#define MISC_WATCHDOG_DIS_MASK       0xFF


/*****************************************************************
 * Interrupts
 *
 * Registers are 8 bits wide.  Masks are active High.  Unused bits read 0
 ****************************************************************/
#define INTERRUPT_BASE               0x78001000

#define INTERRUPT_IRQ0_REG_ID        REG_ID_020
#define INTERRUPT_IRQ0_OFFSET        0x00
#define IRQ0_DMA_INT                 0x1
#define IRQ0_MEZ_INT                 0x2
#define IRQ0_ERNIE_INT               0x4
#define IRQ0_MAC0_PCI_INT            0x8
#define IRQ0_MAC1_PCI_INT            0x10
#define IRQ0_INT_MASK                (IRQ0_DMA_INT | \
                                      IRQ0_MEZ_INT | \
                                      IRQ0_ERNIE_INT | \
                                      IRQ0_MAC0_PCI_INT | \
                                      IRQ0_MAC1_PCI_INT)


#define INTERRUPT_IRQ1_REG_ID        REG_ID_021
#define INTERRUPT_IRQ1_OFFSET        0x04
#define IRQ1_WATCHDOG_INT            0x8
#define IRQ1_IIC_INT                 0x1
#define IRQ1_INT_MASK                (IRQ1_IIC_INT | \
                                      IRQ1_WATCHDOG_INT)


#define INTERRUPT_IRQ2_REG_ID        REG_ID_022
#define INTERRUPT_IRQ2_OFFSET        0x8
#define IRQ2_CUBE_IIC_INT            0x4
#define IRQ2_INT_MASK                (IRQ2_CUBE_IIC_INT)


#define INTERRUPT_IRQ3_REG_ID        REG_ID_023
#define INTERRUPT_IRQ3_OFFSET        0xc
/* CRG - Changed below from 0x4 to 0x2 to match updated BERT Spec */
#define IRQ3_5671_INT                0x2
#define IRQ3_INT_MASK                (IRQ3_5671_INT)

#define INTERRUPT_IRQ4_REG_ID        REG_ID_024
#define INTERRUPT_IRQ4_OFFSET        0x10  /* Unused */
#define IRQ4_INT_MASK                0x0


#define IRQ0_MASK_REG_ID             REG_ID_025
#define IRQ0_MASK_REG_OFFFSET        0x14 /* offset of mask register */
#define IRQ0_MASK_REG_MASK           IRQ0_INT_MASK

#define IRQ1_MASK_REG_ID             REG_ID_026
#define IRQ1_MASK_REG_OFFSET         0x18
#define IRQ1_MASK_REG_MASK           IRQ1_INT_MASK

#define IRQ2_MASK_REG_ID             REG_ID_027
#define IRQ2_MASK_REG_OFFSET         0x1c
#define IRQ2_MASK_REG_MASK           IRQ2_INT_MASK

#define IRQ3_MASK_REG_ID             REG_ID_028
#define IRQ3_MASK_REG_OFFSET         0x20
#define IRQ3_MASK_REG_MASK           IRQ3_INT_MASK

#define IRQ4_MASK_REG_ID             REG_ID_029
#define IRQ4_MASK_REG_OFFSET         0x24
#define IRQ4_MASK_REG_MASK           IRQ4_INT_MASK


/*****************************************************************
 * Reset Defines
 *
 * 8 bit wide registers.  Active High.  Write 1 to reset.
 * Unused bits read 0.
 ****************************************************************/
#define RESET_BASE                      0x78002000

#define GLOBAL_RESET_REG_ID             REG_ID_040
#define GLOBAL_RESET_OFFSET             0x00
#define GLOBAL_GLBL_RST                 0x1
#define GLOBAL_RESET_MASK               (GLOBAL_GLBL_RST)

#define CPU_RESET_REG_ID                REG_ID_041
#define CPU_RESET_OFFSET                0x04
#define CPU_HRST_CPU                    0x1
#define CPU_RESET_MASK                  (CPU_HRST_CPU)

#define CUBE_RESET_REG_ID               REG_ID_042
#define CUBE_RESET_OFFSET               0x08
#define CUBE_CUBE_RST                   0x1
#define CUBE_RESET_MASK                 (CUBE_CUBE_RST)

#define GEA_RESET_REG_ID                REG_ID_043
#define GEA_RESET_OFFSET                0x0C
#define GEA_5671_RST                    0x4
#define GEA_GEA1_RST                    0x2
#define GEA_GEA2_RST                    0x1
#define GEA_RESET_MASK                  (GEA_5671_RST | \
                                         GEA_GEA2_RST | \
                                         GEA_GEA1_RST)

#define PHY_RESET_REG_ID                REG_ID_044
#define PHY_RESET_OFFSET                0x10
#define PHY_PHY0_RST                    0x01
#define PHY_PHY1_RST                    0x02
#define PHY_PHY2_RST                    0x04
#define PHY_PHY3_RST                    0x08
#define PHY_PHY4_RST                    0x10
#define PHY_RESET_MASK                  (PHY_PHY0_RST | \
                                         PHY_PHY1_RST | \
                                         PHY_PHY2_RST | \
                                         PHY_PHY3_RST | \
                                         PHY_PHY4_RST)

#define MEZ_RESET_REG_ID                REG_ID_045
#define MEZ_RESET_OFFSET                0x14
#define MEZ_ERNIE_RST                   0x1
#define MEZ_FC_RST                      0x2
#define MEZ_10G_RST                     0x4
#define MEZ_RESET_MASK                  (MEZ_ERNIE_RST | \
                                         MEZ_FC_RST | \
                                         MEZ_10G_RST)

#define FLASH_RESET_REG_ID              REG_ID_046
#define FLASH_RESET_OFFSET              0x18
#define FLASH_FLASH0_RST                0x1
#define FLASH_FLASH1_RST                0x2
#define FLASH_RESET_MASK                (FLASH_FLASH0_RST | FLASH_FLASH1_RST)

#define LED_DRIVER_RESET_REG_ID         REG_ID_047
#define LED_DRIVER_RESET_OFFSET         0x1c
#define LED_FP_LED_CLR                  0x1
#define LED_CUBE_LED_CLR                0x2
#define LED_DRIVER_RESET_MASK           (LED_FP_LED_CLR | LED_CUBE_LED_CLR)



/************************************************************
 * FRONT LED DEFINES
 ***********************************************************/
#define FRONT_LED_BASE                  0x78003000

/************************************************************
 * Led registers are 4 bits wide
 * Must alternate 0 and 1 to flash LED
 ***********************************************************/
#define LED_DATA_PORT00_REG_ID          REG_ID_050
#define LED_DATA_PORT00_OFFSET          0x00

#define LED_DATA_PORT01_REG_ID          REG_ID_051
#define LED_DATA_PORT01_OFFSET          0x04

#define LED_DATA_PORT02_REG_ID          REG_ID_052
#define LED_DATA_PORT02_OFFSET          0x08

#define LED_DATA_PORT03_REG_ID          REG_ID_053
#define LED_DATA_PORT03_OFFSET          0x0c

#define LED_DATA_PORT04_REG_ID          REG_ID_054
#define LED_DATA_PORT04_OFFSET          0x10

#define LED_DATA_PORT05_REG_ID          REG_ID_055
#define LED_DATA_PORT05_OFFSET          0x14

#define LED_DATA_PORT06_REG_ID          REG_ID_056
#define LED_DATA_PORT06_OFFSET          0x18

#define LED_DATA_PORT07_REG_ID          REG_ID_057
#define LED_DATA_PORT07_OFFSET          0x1c

#define LED_DATA_PORT08_REG_ID          REG_ID_058
#define LED_DATA_PORT08_OFFSET          0x20

#define LED_DATA_PORT09_REG_ID          REG_ID_059
#define LED_DATA_PORT09_OFFSET          0x24

#define LED_DATA_PORT10_REG_ID          REG_ID_060
#define LED_DATA_PORT10_OFFSET          0x28

#define LED_DATA_PORT11_REG_ID          REG_ID_061
#define LED_DATA_PORT11_OFFSET          0x2c

#define LED_DATA_PORT12_REG_ID          REG_ID_062
#define LED_DATA_PORT12_OFFSET          0x30

#define LED_DATA_PORT13_REG_ID          REG_ID_063
#define LED_DATA_PORT13_OFFSET          0x34

#define LED_DATA_PORT14_REG_ID          REG_ID_064
#define LED_DATA_PORT14_OFFSET          0x38

#define LED_DATA_PORT15_REG_ID          REG_ID_065
#define LED_DATA_PORT15_OFFSET          0x3c

#define LED_DATA_PORT16_REG_ID          REG_ID_066
#define LED_DATA_PORT16_OFFSET          0x40

#define LED_DATA_PORT17_REG_ID          REG_ID_067
#define LED_DATA_PORT17_OFFSET          0x44

#define LED_DATA_PORT18_REG_ID          REG_ID_068
#define LED_DATA_PORT18_OFFSET          0x48

#define LED_DATA_PORT19_REG_ID          REG_ID_069
#define LED_DATA_PORT19_OFFSET          0x4c

#define LED_DATA_PORT20_REG_ID          REG_ID_070
#define LED_DATA_PORT20_OFFSET          0x50

#define LED_DATA_PORT21_REG_ID          REG_ID_071
#define LED_DATA_PORT21_OFFSET          0x54

#define LED_DATA_PORT22_REG_ID          REG_ID_072
#define LED_DATA_PORT22_OFFSET          0x58

#define LED_DATA_PORT23_REG_ID          REG_ID_073
#define LED_DATA_PORT23_OFFSET          0x5c


#define LED_LINK_MASK                   0xC
#define LED_SPEED_MASK                  0x3
#define LED_DATA_MASK                   (LED_LINK_MASK | LED_SPEED_MASK)

#define DATA_LINK_LED_OFF               0x0  /* 00xx */ /* No Link */
#define DATA_LINK_GREEN                 0x4  /* 01xx */ /* Link    */
#define DATA_LINK_AMBER                 0x8  /* 10xx */ /* Admin down */
#define DATA_LINK_LED_OFF2              0xC  /* 11xx */ /* No Link */

#define DATA_SPEED_LED_OFF              0x0  /* xx00 */ /* 10 Base-T */
#define DATA_SPEED_GREEN                0x1  /* xx01 */ /* 100 Base-T */
#define DATA_SPEED_AMBER                0x2  /* xx10 */ /* 1000 Base-T */
#define DATA_SPEED_LED_OFF2             0x3  /* xx11 */ /* 10 Base-T */

/* Management and Mismatch */
#define LED_DATA_MANAGEMENT_REG_ID      REG_ID_074
#define LED_DATA_MANAGEMENT_OFFSET      0x60
#define LED_DATA_MANAGEMENT_MASK        0x3
#define DATA_MGMNT_LED_OFF              0x0  /* xx00 */
#define DATA_MGMNT_GREEN                0x1  /* xx01 */
#define DATA_MGMNT_AMBER                0x2  /* xx10 */
#define DATA_MGMNT_LED_OFF2             0x3  /* xx11 */ 

#define LED_DATA_MISMATCH_REG_ID        REG_ID_075
#define LED_DATA_MISMATCH_OFFSET        0x64
#define LED_MISMATCH_CFG_MASK           0xC
#define LED_MISMATCH_SWITCH_MASK        0x3
#define LED_DATA_MISMATCH_MASK          (LED_MISMATCH_SWITCH_MASK | \
                                         LED_MISMATCH_CFG_MASK)
#define DATA_CFG_LED_OFF                0x0  /* 00xx */ 
#define DATA_CFG_GREEN                  0x4  /* 01xx */
#define DATA_CFG_AMBER                  0x8  /* 10xx */ 
#define DATA_CFG_LED_OFF2               0xC  /* 11xx */

#define DATA_SWITCH_LED_OFF             0x0  /* xx00 */ 
#define DATA_SWITCH_GREEN               0x1  /* xx01 */
#define DATA_SWITCH_AMBER               0x2  /* xx10 */ 
#define DATA_SWITCH_LED_OFF2            0x3  /* xx11 */

/************************************************************
 * Registers are 8 bits wide, Active high
 * Led Data Mask 0x6c corresponds to ports 0 - 7
 *     bit 0=port 0, bit 1=port 1, etc
 * Led Data Mask 0x70 corresponds to ports 8 - 15
 *     bit 0=port 8, bit 1=port 9, etc
 * 0x6c = 0xF7 will disable port 3's LED.
 ***********************************************************/
#define LED_MASK_PORTS00_07_REG_ID      REG_ID_076
#define LED_MASK_PORTS00_07_OFFSET      0x80

#define LED_MASK_PORTS08_15_REG_ID      REG_ID_077
#define LED_MASK_PORTS08_15_OFFSET      0x84

#define LED_MASK_PORTS16_23_REG_ID      REG_ID_078
#define LED_MASK_PORTS16_23_OFFSET      0x88

#define LED_MASK_SIZE                   0x8 /* 8 bit register */
#define LED_MASK_PORTS_REG_MASK         0xFF

#define LED_10G_MEZ_SEL_REG_ID          REG_ID_079
#define LED_10G_MEZ_SEL_OFFSET          0x8c
#define LED_ENABLE_MEZ_LED              0x1
#define LED_10G_MEZ_SEL_MASK            (LED_ENABLE_MEZ_LED)






#endif /* GLUEBERT_H */
#endif /* CONFIG_BERT */
/* End of File */
